<?php
namespace net\xmp\phpbuch;

class HTMLPage
{
    protected $title;
    protected $content;
    protected $template;

    function __construct($title, $template='./template.html')
    {
        $this->title = $title;
        $this->content = '';
        $this->template = $template;
    }
    
    /**
     * Gibt erstellte HTML-Seite aus.
     */
    function printPage()
    {
        global $page;
        $page = $this;
        require $this->template;
    }

    /**
     * Gibt einzelne Datenfelder von HTMLPage zurück.
     *
     * @param string $name Name des Datenfelds
     * @param boolean $escape Gibt an, ob Wert für HTML-Ausgabe maskiert wird
     */
    function get($name, $escape=true)
    {
        if ($escape) {
            return $this->escape($this->$name);
        }
        else {
            return $this->$name;
        }
    }

    /**
     * Fügt der Seite ein HTML-Element mit Textinhalt hinzu.
     *
     * @param string $name Name des HTML-Elements
     * @param string $content Inhalt des Elements 
     */
    function addElement($name, $content)
    {
        $this->content .= "<$name>" . $this->escape($content) . "</$name>";
    }

    /**
     * Fügt der Seite ungefiltertes HTML hinzu.
     *
     * @param string $html HTML, das hinzugefügt werden soll
     */
    function addHTML($html)
    {
        $this->content .= $html;
    }

    /**
     * Fügt der Seite eine Tabelle hinzu.
     * 
     * @param array $content Zweidimensionales Array:
     *                       Erste Zeile enthält Überschriften (<th>)
     *                       Folgende Zeilen enthalten Daten (<td>)
     * @param array $rawHTML Gibt an, welche Spalten als ungefiltertes HTML
     *                       hinzugefügt werden sollen
     */
    function addTable($content, $rawHTML=array())
    {
        $this->content .= "<table>\n";
        // Header hinzufügen
        $this->content .= "<thead><tr>\n";
        foreach ($content[0] as $cell) {
            $this->addElement('th', $cell);
        }
        $this->content .= "</tr></thead>\n";
        // Daten hinzufügen
        $this->content .= '<tbody>';
        for ($i = 1; $i < count($content); $i++) {
            $this->content .= '<tr>';
            for ($j = 0; $j < count($content[$i]); $j++) {
                if (empty($rawHTML[$j])) {
                    $this->addElement('td', $content[$i][$j]);
                }
                else {
                    $this->addHTML('<td>' . $content[$i][$j] . '</td>');
                }
            }
            $this->content .= "</tr>\n";
        }
        $this->content .= "</tbody></table>\n";
    }

    /**
     * Fügt der Seite eine Liste hinzu.
     * 
     * @param array $items Einträge in Liste. Falls Eintrag ein Array ist, wird erster Teil
     *                     hervorgehoben und zweiter Teil normal ausgegeben 
     * @param boolean $rawHTML Gibt an, ob die Einträge ungefiltert eingefügt werden sollen.
     */
    function addList($items, $rawHTML=false)
    {
        $this->content .= "<ul>\n";
        foreach ($items as $item) {
            if (!is_array($item)) {
                $rawHTML ? $this->addHTML('<li>' . $item . '</li>')
                         : $this->addElement('li', $item);
            }
            else {
                $this->content .= '<li>';
                $this->addElement('b', $item[0] . ': ');
                $this->content .= $rawHTML ? $item[1]
                                           : $this->escape($item[1]);
                $this->content .= '</li>';
            }
        }
        $this->content .= "</ul>\n";
    }

    /**
     * Maskieren von Texten zur Ausgabe auf HTML-Seite
     * Löscht alle Zeichen kleiner chr(32), ersetzt kritische Zeichen durch Entities
     *
     * @param string $txt Text
     */
    function escape($txt)
    {
        $txt = preg_replace('/\s+/', ' ', $txt);
        return filter_var($txt, FILTER_SANITIZE_SPECIAL_CHARS, FILTER_FLAG_STRIP_LOW);
    }
}
