<?php
/* Vollständig zusammengesetztes Listing */
namespace net\xmp\phpbuch;

class LDAPConnection
{
    public $handle = null;
    public $baseDN = '';

    /**
     * Verbindung mit LDAP-Verzeichnis herstellen.
     *
     * @param string $server Name des LDAP-Servers
     * @param string $username Benutzername
     * @param string $password Passwort
     * @param boolean $secure Gibt an, ob eine TLS-verschlüsselte Verbindung verwendet werden soll
     */
    function connect($server, $userName, $password, $secure=false)
    {
        $this->handle = ldap_connect($server);
        ldap_set_option($this->handle, LDAP_OPT_PROTOCOL_VERSION, 3);
        ldap_set_option($this->handle, LDAP_OPT_REFERRALS, 0);
        if ($secure) {
            if (!ldap_start_tls($this->handle)) {
                $this->exitWithError('Start-TLS fehlgeschlagen');
            }
        }
        if (!ldap_bind($this->handle, $userName, $password)) {
            $this->exitWithError('Bindung der LDAP-Verbindung fehlgeschlagen');
        }
    }

    /**
     * Verbindung zu LDAP-Verzeichnis schließen.
     */
    function close()
    {
        if ($this->handle) {
            ldap_unbind($this->handle);
            $this->handle = null;
        }
    }
    
    /**
     * Programm beenden, Fehlermeldung von LDAP-Verzeichnis ausgeben
     * 
     * @param string $txt Beschreibung des Fehlerkontexts
     */
    function exitWithError($txt)
    {
        $errors = ldap_error();
        $html = new HTMLPage('LDAP-Fehler');
        $html->addElement('p', $txt);
        $table = array( array('Nummer', 'Meldung'),
                        array(ldap_errno($this->handle), ldap_error($this->handle)));
        $html->addTable($table);
        $html->printPage();
        $this->close();
        exit;
    }

    /**
     * Auslesen eines Eintrages mit SAM-Accountname
     * @param string $account SAM-Accountname
     * @param array $attributes auszulesende Attribute
     * @param string $baseDN Basis-DN für Suche
     */
    function getAccount($account, $attributes=array(), $baseDN='')
    {
        $filter = '(sAMAccountName=' . escapeLDAP($account) . ')';
        $account = $this->getOneEntry($filter, $attributes, $baseDN);
        if (!$account) {
            $this->exitWithError('Account nicht gefunden.');
        }
        return $account;
    }

    /**
     * Auslesen eines Eintrages einer Suche
     * @param string $filter Filterzeichenfolge
     * @param array $attributes auszulesende Attribute
     * @param string $baseDN Basis-DN für Suche
     */
    function getOneEntry($filter, $attributes=array(), $baseDN='')
    {
        if (empty($baseDN)) {
            $baseDN = $this->baseDN;
        }
        $result = ldap_search($this->handle, $baseDN, $filter, $attributes);
        if (!$result) {
            return false;
        }
        $entry = ldap_first_entry($this->handle, $result);
        if (!$entry) {
            return false;
        }
        $attr = ldap_get_attributes($this->handle, $entry);
        if (!$attr) {
            return false;
        }
        $attr['dn'] = ldap_get_dn($this->handle, $entry);
        return $attr;
    }
}
?>
